<?php
/**
 * A command for the 'apt-proxy client' routine
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2011 - 2012 Apt-Proxy Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class HubConsoleAptProxyCommand extends BaseCommand implements Commandable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$resolverInstance	An instance of a command resolver class
	 * @return	$commandInstance	An instance a prepared command class
	 */
	public static final function createHubConsoleAptProxyCommand (CommandResolver $resolverInstance) {
		// Get new instance
		$commandInstance = new HubConsoleAptProxyCommand();

		// Set the application instance
		$commandInstance->setResolverInstance($resolverInstance);

		// Return the prepared instance
		return $commandInstance;
	}

	/**
	 * Executes the given command with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 * @todo	Try to create a AptProxyActivationTask or so
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get a registry and the application instance from it
		$applicationInstance = Registry::getRegistry()->getInstance('app');

		/*
		 * ----------------------- Bootstrapping phase ------------------------
		 * Try to bootstrap the node and pass the request instance to it for
		 * extra arguments which mostly override config entries or enable special
		 * features within the hub (none is ready at this development stage)
		 */
		self::createDebugInstance(__CLASS__)->debugOutput('BOOTSTRAP: Beginning with bootstrap...');
		$applicationInstance->getControllerInstance()->executeBootstrapFilters($requestInstance, $responseInstance);
		self::createDebugInstance(__CLASS__)->debugOutput('BOOTSTRAP: Bootstrap finished.');

		// Get apt-proxy instance
		$proxyInstance = Registry::getRegistry()->getInstance('apt_proxy');

		// Add some apt-proxy-specific filters
		$proxyInstance->addExtraAptProxyFilters();

		// Get task handler instance
		$handlerInstance = Registry::getRegistry()->getInstance('task_handler');

		// Debug message
		self::createDebugInstance(__CLASS__)->debugOutput('MAIN: --- Entering main loop. ---');

		/*
		 * ----------------------------- Main loop ----------------------------
		 * This is the main loop. Queried calls should come back here very fast
		 * so the whole application runs on nice speed. This while-loop goes
		 * until the apt-proxy is no longer active or all tasks are killed.
		 */
		while (($proxyInstance->isActive()) && ($handlerInstance->hasTasksLeft())) {
			// Handle all tasks here
			$handlerInstance->handleTasks();
		} // END - while

		// Debug message
		self::createDebugInstance(__CLASS__)->debugOutput('MAIN: --- Leaving main loop. ---');
	}

	/**
	 * Adds extra filters to the given controller instance
	 *
	 * @param	$controllerInstance		A controller instance
	 * @param	$requestInstance		An instance of a class with an Requestable interface
	 * @return	void
	 * @todo	Should we add some more filters?
	 */
	public function addExtraFilters (Controller $controllerInstance, Requestable $requestInstance) {
		// Add pre filters
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_php_requirements_filter'));
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_initializer_filter'));
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_welcome_teaser_filter'));

		// Add bootstrap filters
		$controllerInstance->addBootstrapFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_bootstrap_task_handler_initializer_filter'));
		$controllerInstance->addBootstrapFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_bootstrap_extra_bootstrapping_filter'));

		// This is the last generic boostrap filter
		$controllerInstance->addBootstrapFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_bootstrap_generic_activation_filter'));

		// Add shutdown filters
		//$controllerInstance->addShutdownFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_shutdown_task_handler_filter'));

		// This is the last generic shutdown filter
		$controllerInstance->addShutdownFilter(ObjectFactory::createObjectByConfiguredName('apt_proxy_shutdown_filter'));
	}
}

// [EOF]
?>
