<?php
/**
 * A NodeMessageDhtBootstrap handler
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class NodeMessageDhtBootstrapHandler extends BaseMessageHandler implements HandleableMessage, Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set handler name
		$this->setHandlerName('message_dht_bootstrap');

		// Init message data array
		$this->messageDataElements = array(
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_EXTERNAL_ADDRESS,
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_INTERNAL_ADDRESS,
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_NODE_STATUS,
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_NODE_MODE,
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_SESSION_ID,
		);

		// Init message-data->configuration translation array
		$this->messageToConfig = array(
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_EXTERNAL_ADDRESS => 'your_external_address',
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_INTERNAL_ADDRESS => 'your_internal_address',
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_SESSION_ID       => 'your_session_id'
		);

		// Init config-copy array
		$this->configCopy = array(
			XmlDhtBootstrapAnswerTemplateEngine::DHT_BOOTSTRAP_DATA_EXTERNAL_ADDRESS => 'external_address',
			XmlDhtBootstrapAnswerTemplateEngine::DHT_BOOTSTRAP_DATA_INTERNAL_ADDRESS => 'internal_address',
			XmlDhtBootstrapAnswerTemplateEngine::DHT_BOOTSTRAP_DATA_NODE_STATUS      => 'node_status',
			XmlDhtBootstrapAnswerTemplateEngine::DHT_BOOTSTRAP_DATA_SESSION_ID       => 'session_id',
		);

		// Init array
		$this->searchData = array(
			XmlDhtBootstrapTemplateEngine::DHT_BOOTSTRAP_DATA_EXTERNAL_ADDRESS,
		);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$handlerInstance	An instance of a HandleableMessage class
	 */
	public final static function createNodeMessageDhtBootstrapHandler () {
		// Get new instance
		$handlerInstance = new NodeMessageDhtBootstrapHandler();

		// Get a DHT instance
		$dhtInstance = DhtObjectFactory::createDhtInstance('node');

		// Set the DHT instance here
		$handlerInstance->setDhtInstance($dhtInstance);

		// Return the prepared instance
		return $handlerInstance;
	}

	/**
	 * Handles data array of the message
	 *
	 * @param	$messageData		An array with message data to handle
	 * @param	$packageInstance	An instance of a Receivable class
	 * @return	void
	 * @throws	DhtBootstrapNotAcceptedException	If this node does not accept DHT bootstrap requests
	 */
	public function handleMessageData (array $messageData, Receivable $packageInstance) {
		// Is this node accepting DHT bootstrap requests?
		if (!NodeObjectFactory::createNodeInstance()->isAcceptingDhtBootstrap()) {
			/*
			 * This node is not accepting DHT bootstrap requests.
			 */
			throw new DhtBootstrapNotAcceptedException(array($this, $messageData), self::EXCEPTION_DHT_BOOTSTRAP_NOT_ACCEPTED);
		} // END - if

		// Register the DHT bootstrap requesting node with this node
		$this->registerNodeByMessageData($messageData);

		// Prepare answer message to be delivered back to the other node
		$this->prepareAnswerMessage($messageData, $packageInstance);
	}

	/**
	 * Adds all required elements from given array into data set instance
	 *
	 * @param	$dataSetInstance	An instance of a StoreableCriteria class
	 * @param	$messageData		An array with all message data
	 * @return	void
	 */
	public function addArrayToDataSet (StoreableCriteria $dataSetInstance, array $messageData) {
		// Add generic first
		parent::addArrayToDataSet($dataSetInstance, $messageData);

		// Add all ements
		foreach ($this->messageDataElements as $key) {
			// Is it there?
			assert(isset($messageData[$key]));

			// Add it
			$dataSetInstance->addCriteria($key, $messageData[$key]);
		} // END - foreach
	}

	/**
	 * Initializes configuration data from given message data array
	 *
	 * @param	$messageData	An array with all message data
	 * @return	void
	 */
	protected function initMessageConfigurationData (array $messageData) {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DHT-BOOTSTRAP-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: messageData=' . print_r($messageData, TRUE));

		// "Walk" throught the translation array
		foreach ($this->messageToConfig as $messageKey => $configKey) {
			// Debug message
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DHT-BOOTSTRAP-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: Setting messageKey=' . $messageKey . ',configKey=' . $configKey . ':' . $messageData[$messageKey]);

			// Set the element in configuration
			$this->getConfigInstance()->setConfigEntry($configKey, $messageData[$messageKey]);
		} // END - foreach

		// "Walk" throught the config-copy array
		foreach ($this->configCopy as $targetKey => $sourceKey) {
			// Debug message
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DHT-BOOTSTRAP-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: Copying from sourceKey=' . $sourceKey . ' to targetKey=' . $targetKey . '...');

			// Copy from source to target key
			$this->getConfigInstance()->setConfigEntry($targetKey, $this->getConfigInstance()->getConfigEntry($sourceKey));
		} // END - foreach

		// Translate last exception into a status code
		$statusCode = $this->getTranslatedStatusFromLastException();

		// Set it in configuration (temporarily)
		$this->getConfigInstance()->setConfigEntry(NodeDistributedHashTableDatabaseWrapper::DB_COLUMN_ANSWER_STATUS, $statusCode);

		/*
		 * Use the DHT instance to get a list of recipients. This means that all
		 * DHT nodes that accept bootstrap requests are read from the DHT
		 * database.
		 */
		$nodeList = $this->getDhtInstance()->findRecipientsByKey(NodeDistributedHashTableDatabaseWrapper::DB_COLUMN_ACCEPT_BOOTSTRAP, 'Y');

		// Make sure it is an array and has at least one entry
		assert(is_array($nodeList));
		assert(count($nodeList) > 0);

		// Set it in configuration
		$this->getConfigInstance()->setConfigEntry('dht_nodes', base64_encode(json_encode($nodeList)));
	}

	/**
	 * Removes configuration data with given message data array from global
	 * configuration
	 *
	 * @param	$messageData	An array with all message data
	 * @return	void
	 */
	protected function removeMessageConfigurationData (array $messageData) {
		// "Walk" throught the translation array again
		foreach ($this->messageToConfig as $dummy => $configKey) {
			// Now unset this configuration entry (to save some memory)
			$this->getConfigInstance()->unsetConfigEntry($configKey);
		} // END - foreach

		// "Walk" throught the config-copy array again
		foreach ($this->configCopy as $configKey => $dummy) {
			// Now unset this configuration entry (to save some memory again)
			$this->getConfigInstance()->unsetConfigEntry($configKey);
		} // END - foreach

		// Remove temporary "special" values as well
		$this->getConfigInstance()->unsetConfigEntry(NodeDistributedHashTableDatabaseWrapper::DB_COLUMN_ANSWER_STATUS);
		$this->getConfigInstance()->unsetConfigEntry('dht_nodes');
	}
}

// [EOF]
?>
