<?php
/**
 * A default peer pool class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class DefaultPeerPool extends BasePool implements PoolablePeer {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$listenerInstance	A Listenable instance
	 * @return	$poolInstance		An instance a Poolable class
	 */
	public static final function createDefaultPeerPool (Listenable $listenerInstance) {
		// Get new instance
		$poolInstance = new DefaultPeerPool();

		// Set the application instance
		$poolInstance->setListenerInstance($listenerInstance);

		// Return the prepared instance
		return $poolInstance;
	}

	/**
	 * Validates given socket
	 *
	 * @param	$socketResource		A valid socket resource
	 * @return	void
	 * @throws	InvalidSocketException	If the given socket has an error
	 */
	private function validateSocket ($socketResource) {
		// Is it a valid resource?
		if (!is_resource($socketResource)) {
			// Throw an exception
			throw new InvalidSocketException(array($this, $socketResource), BaseListener::EXCEPTION_INVALID_SOCKET);
		} // END - if

		// Get error code
		$errorCode = socket_last_error($socketResource);

		// Is it without any errors?
		if ($errorCode > 0) {
			// Handle the socket error with a faked recipientData array
			$this->handleSocketError(__METHOD__, __LINE__, $socketResource, array('0.0.0.0', '0'));
			/*
			// Get error message
			$errorMessage = socket_strerror($errorCode);

			// Shutdown this socket
			$this->getListenerInstance()->shutdownSocket($socketResource);

			// And throw again
			throw new InvalidSocketException(array($this, $socketResource, $errorCode, $errorMessage), BaseListener::EXCEPTION_INVALID_SOCKET);
			*/
		} // END - if
	}

	/**
	 * Adds a socket resource to the peer pool
	 *
	 * @param	$socketResource		A valid (must be!) socket resource
	 * @param	$connectionType		Type of connection, can only be 'incoming', 'outgoing' or 'server'
	 * @return	void
	 * @throws	InvalidSocketException	If the given resource is invalid or errorous
	 * @throws	InvalidConnectionTypeException	If the provided connection type is not valid
	 */
	public function addPeer ($socketResource, $connectionType) {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DEFAULT-PEER-POOL[' . __METHOD__ . ':' . __LINE__ . ']: socketResource[' . gettype($socketResource) . ']=' . $socketResource . ',connectionType=' . $connectionType . ' - CALLED!');

		// Validate the socket
		$this->validateSocket($socketResource);

		// Is the connection type valid?
		if (!$this->isValidConnectionType($connectionType)) {
			// Is not a valid connection type!
			throw new InvalidConnectionTypeException(array($this, $connectionType), self::EXCEPTION_INVALID_CONNECTION_TYPE);
		} // END - if

		// Default is this peer's IP
		$peerName = '0.0.0.0';

		// The socket resource should not match server socket
		if ($socketResource != $this->getListenerInstance()->getSocketResource()) {
			// Try to determine the peer's IP number
			if (!socket_getpeername($socketResource, $peerName)) {
				// Handle the socket error with a faked recipientData array
				$this->handleSocketError(__METHOD__, __LINE__, $socketResource, array('0.0.0.0', '0'));
				/*
				// Get last error
				$lastError = socket_last_error($socketResource);

				// Doesn't work!
				throw new InvalidSocketException(array($this, $socketResource, $lastError, socket_strerror($lastError)), BaseListener::EXCEPTION_INVALID_SOCKET);
				*/
			} // END - if
		} else {
			// Server sockets won't work with socket_getpeername()
			self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: Socket resource is server socket (' . $socketResource . '). This is not a bug.');
		}

		// Debug message
		self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: Adding peer ' . $peerName . ',socketResource=' . $socketResource . ',type=' . $connectionType);

		// Construct the array
		$socketArray = array(
			self::SOCKET_ARRAY_RESOURCE  => $socketResource,
			self::SOCKET_ARRAY_CONN_TYPE => $connectionType
		);

		// Add it finally to the pool
		$this->addPoolEntry($socketArray);
	}

	/**
	 * Getter for array of all socket resources
	 *
	 * @return	$sockets	An array with all sockets
	 */
	public final function getAllSockets () {
		// Get the list
		$sockets = $this->getArrayFromList('pool');

		// Return it
		return $sockets;
	}

	/**
	 * Getter for array of all socket arrays
	 *
	 * @return	$sockets	An array with all socket arrays
	 */
	public final function getAllSingleSockets () {
		// Get the array list
		$socketArrays = $this->getArrayFromList('pool');

		// Init socket array
		$sockets = array();

		// "Walk" through all socket arrays
		foreach ($socketArrays as $socketArray) {
			// Add the socket
			array_push($sockets, $socketArray[self::SOCKET_ARRAY_RESOURCE]);
		} // END - foreach

		// Return it
		return $sockets;
	}

	/**
	 * "Getter" for all sockets of specified type
	 *
	 * @param	$connectionType		Type of connection, can only be 'incoming', 'outgoing' or 'server'
	 * @return	$sockets			An array with sockets of given type
	 * @throws	InvalidConnectionTypeException	If the provided connection type is not valid
	 */
	public function getSocketsByConnectionType ($connectionType) {
		// Is the connection type valid?
		if (!$this->isValidConnectionType($connectionType)) {
			// Is not a valid connection type!
			throw new InvalidConnectionTypeException(array($this, $connectionType), self::EXCEPTION_INVALID_CONNECTION_TYPE);
		} // END - if

		// Get the array list
		$socketArrays = $this->getArrayFromList('pool');

		// Init socket array
		$sockets = array();

		// "Walk" through all socket arrays
		foreach ($socketArrays as $socketArray) {
			// Does it match?
			if ($socketArray[self::SOCKET_ARRAY_CONN_TYPE] === $connectionType) {
				// Add the socket
				array_push($sockets, $socketArray[self::SOCKET_ARRAY_RESOURCE]);
			} // END - if
		} // END - foreach

		// Return it
		return $sockets;
	}

	/**
	 * "Getter" for a valid socket resource from given packae data.
	 *
	 * @param	$packageData		Raw package data
	 * @param	$connectionType		Type of connection, can be 'incoming', 'outgoing', 'server' or default
	 * @return	$socketResource		Socket resource
	 * @throws	InvalidConnectionTypeException	If the provided connection type is not valid
	 */
	public function getSocketFromPackageData (array $packageData, $connectionType = NULL) {
		// Default is no socket
		$socketResource = FALSE;

		// Resolve recipient (UNL) into a handler instance
		$handlerInstance = ProtocolHandlerFactory::createProtocolHandlerFromPackageData($packageData);

		// Get UNL data
		$unlData = $handlerInstance->getUniversalNodeLocatorDataArray();

		// Make sure it is a valid Universal Node Locator array (3 elements)
		assert(isset($unlData[UniversalNodeLocator::UNL_PART_PROTOCOL]));
		assert(isset($unlData[UniversalNodeLocator::UNL_PART_ADDRESS]));
		assert(isset($unlData[UniversalNodeLocator::UNL_PART_PORT]));

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: Checking ' . count($this->getAllSockets()) . ' socket(s),unlData[' . UniversalNodeLocator::UNL_PART_ADDRESS . ']=' . $unlData[UniversalNodeLocator::UNL_PART_ADDRESS] . ',unlData[' . UniversalNodeLocator::UNL_PART_PORT . ']=' . $unlData[UniversalNodeLocator::UNL_PART_PORT] . ' ...');

		// Default is all sockets
		$sockets = $this->getAllSockets();

		// Is connection type set?
		if ((is_string($connectionType)) && ($this->isValidConnectionType($connectionType))) {
			// Then get a list of this type
			$sockets = $this->getSocketsByConnectionType($connectionType);
		} elseif (is_string($connectionType)) {
			// Is not a valid connection type!
			throw new InvalidConnectionTypeException(array($this, $connectionType), self::EXCEPTION_INVALID_CONNECTION_TYPE);
		}

		// Get all sockets and check them, skip the server socket
		foreach ($sockets as $socketArray) {
			// Is this a server socket?
			if ($socketArray[self::SOCKET_ARRAY_RESOURCE] === $this->getListenerInstance()->getSocketResource()) {
				// Skip 'server' sockets (local socket)
				//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: Skipping server socket ' . $socketArray[self::SOCKET_ARRAY_RESOURCE] . ' ...');
				continue;
			} // END - if

			// Try to get the "peer"'s name
			if (!socket_getpeername($socketArray[self::SOCKET_ARRAY_RESOURCE], $peerIp)) {
				// Handle the socket error with given package data
				$this->handleSocketError(__METHOD__, __LINE__, $socketArray[self::SOCKET_ARRAY_RESOURCE], explode(':', $packageData[NetworkPackage::PACKAGE_DATA_RECIPIENT]));
			} // END - if

			// Get
			// If the "peer" IP and recipient is same, use it
			if ($peerIp == $unlData[UniversalNodeLocator::UNL_PART_ADDRESS]) {
				// IPs match, so take the socket and quit this loop
				$socketResource = $socketArray[self::SOCKET_ARRAY_RESOURCE];

				// Debug message
				//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: peerIp=' . $peerIp . ' matches with recipient IP address. Taking socket=' . $socketArray[self::SOCKET_ARRAY_RESOURCE] . ',type=' . $socketArray[self::SOCKET_ARRAY_CONN_TYPE]);
				break;
			} // END - if
		} // END - foreach

		// Return the determined socket resource
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: socketResource[' . gettype($socketResource) . ']=' . $socketResource);
		return $socketResource;
	}
}

// [EOF]
?>
