<?php
/**
 * A TCP protocol resolver class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class TcpProtocolResolver extends BaseProtocolResolver implements ProtocolResolver, Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set protocol type
		$this->setProtocolName('tcp');
	}

	/**
	 * Creates an instance of a TCP protocol resolver
	 *
	 * @return	$resolverInstance	The prepared command resolver instance
	 */
	public static final function createTcpProtocolResolver () {
		// Create the new instance
		$resolverInstance = new TcpProtocolResolver();

		// Return the prepared instance
		return $resolverInstance;
	}

	/**
	 * Returns an instance of a LocateableNode class for a given NodeHelper
	 * instance or null if it was not found.
	 *
	 * @param	$nodeInstance	An instance of a NodeHelper class
	 * @return	$unlInstance	An instance of a LocateableNode class
	 * @todo	0% done
	 */
	public function resolveUniversalNodeLocatorFromNodeHelper (NodeHelper $nodeInstance) {
		// Get search instance (to lookup database result)
		$searchInstance = $nodeInstance->getSearchInstance();

		// Make sure the in stance is valid
		assert($searchInstance instanceof SearchCriteria);

		// Refetch and rewind iterator
		$resultInstance = $nodeInstance->getWrapperInstance()->doSelectByCriteria($searchInstance);

		// Is the result valid?
		if ((!$resultInstance->valid()) || (!$resultInstance->next())) {
			// Node not found in database, this could mean that your database file is damaged.
			return NULL;
		} // END - if

		// Get current entry
		$current = $resultInstance->current();

		// This should always be the case, if not your database file might be damaged.
		assert($nodeInstance->getNodeId() == $current[NodeInformationDatabaseWrapper::DB_COLUMN_NODE_ID]);

		// Get UNL instance and handle over all data
		$unlInstance = ObjectFactory::createObjectByConfiguredName('universal_node_locator_class', array($current));

		// Return resolved instance
		return $unlInstance;
	}

	/**
	 * Returns the UNL as string from given configuration key.
	 *
	 * @param	$configKey	Configuration key for UNL address (valid: internal,external)
	 * @return	$unl		Universal node locator
	 */
	public function resolveUniversalNodeLocatorFromConfigKey ($configKey) {
		// Get address
		$address = $this->getConfigInstance()->getConfigEntry($configKey . '_address');

		// Is the address empty?
		if (empty($address)) {
			// Okay, then find it
			switch ($configKey) {
				case 'external': // External IP
					$address = ConsoleTools::determineExternalAddress();
					break;

				case 'internal': // Internal IP
					$address = ConsoleTools::acquireSelfIPAddress();
					break;
			} // END - switch
		} // END - if

		// Put all together
		$unl = sprintf('%s://%s:%s',
			$this->getProtocolName(),
			$address,
			$this->getConfigInstance()->getConfigEntry('node_listen_port')
		);

		/*
		 * And return it. Please note that e.g. a FaxProtocolResolver will
		 * return a different UNL and therefore all protocol resolvers must do
		 * it on their own way.
		 */
		return $unl;
	}
}

// [EOF]
?>
